==> openiec61850-0.11.0.tgz: openiec61850/src/common/src/main/java/org/openiec61850/common/model/ObjectReference.java <==
/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * ObjectReference syntax: LDName/LNName.DOName[.Name[. ...]]
 * 
 * @author Michael Zillgith
 * @author Tobias Weidelt
 * @author Stefan Feuerhahn
 * 
 */
public class ObjectReference implements Iterable<String> {

	private final String objectReference;
	private List<String> nodeNames = null;
	private String nameCache = null;

	public ObjectReference(String objectReference) {
		if (objectReference == null || objectReference == "") {
			throw new IllegalArgumentException();
		}
		this.objectReference = objectReference;
	}

	/**
	 * @return Returns name part of the reference.
	 */
	public String getName() {
		if (nameCache != null) {
			return nameCache;
		}

		int firstSlash = objectReference.indexOf('/');

		if (firstSlash >= 0) {
			int lastDot = objectReference.lastIndexOf('.');
			if (lastDot > firstSlash) {
//				int lastSqrBracket = objectReference.lastIndexOf('[');
//				if (lastSqrBracket != -1) {//this is for nested values
//					if (lastSqrBracket > lastDot) {
//						int lastSqrClosingBracket = objectReference
//								.lastIndexOf(']');
//						nameCache = objectReference.substring(
//								lastSqrBracket + 1, lastSqrClosingBracket);
//						return nameCache;
//					}
//				} else {// this is for arrays
					int lastBracket = objectReference.lastIndexOf('(');
					if (lastBracket > lastDot) {
						int lastClosingBracket = objectReference
								.lastIndexOf(')');
						nameCache = objectReference.substring(
								lastBracket + 1, lastClosingBracket);
						return nameCache;
					}
//				}
				nameCache = objectReference.substring(lastDot + 1);
				return nameCache;

			}
			nameCache = objectReference.substring(firstSlash + 1);
			return nameCache;
		}
		nameCache = objectReference;
		return nameCache;
	}

	@Override
	public String toString() {
		return objectReference;
	}

	public boolean isLogicalDeviceRef() {
		return (size() == 1);
	}

	public boolean isLogicalNodeRef() {
		return (size() == 2);
	}

	public boolean isDataRef() {
		return (size() > 2);
	}

	public Iterator<String> iterator() {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.iterator();
	}

	public String get(int i) {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.get(i);
	}

	public int size() {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.size();
	}

	public String getParentRef() {
		int lastDot = objectReference.lastIndexOf('.');
		if (lastDot >= 0) {
			return objectReference.substring(0, lastDot);
		}
		int lastSlash = objectReference.lastIndexOf('/');
		if (lastSlash >= 0) {
			return objectReference.substring(0, lastSlash);
		}
		return "";
	}

	private void parseForNameList() {

		nodeNames = new LinkedList<String>();

		int lastDelim = -1;
		int nextDelim = objectReference.indexOf('/');
		if (nextDelim == -1) {
			nodeNames.add(objectReference.substring(lastDelim + 1,
					objectReference.length()));
			return;
		}

		nodeNames.add(objectReference.substring(lastDelim + 1, nextDelim));

		int dotIndex = -1;
		int openingbracketIndex = -1;
		int closingbracketIndex = -1;
		while (true) {
			lastDelim = nextDelim;
			if (dotIndex == -1) {
				dotIndex = objectReference.indexOf('.', lastDelim + 1);
				if (dotIndex == -1) {
					dotIndex = objectReference.length();
				}
			}
			if (openingbracketIndex == -1) {
				openingbracketIndex = objectReference.indexOf('(',
						lastDelim + 1);
				if (openingbracketIndex == -1) {
					openingbracketIndex = objectReference.length();
				}
			}
			if (closingbracketIndex == -1) {
				closingbracketIndex = objectReference.indexOf(')',
						lastDelim + 1);
				if (closingbracketIndex == -1) {
					closingbracketIndex = objectReference.length();
				}
			}

			if (dotIndex == openingbracketIndex
					&& dotIndex == closingbracketIndex) {
				nodeNames.add(objectReference.substring(lastDelim + 1,
						objectReference.length()));
				return;
			}

			if (dotIndex < openingbracketIndex
					&& dotIndex < closingbracketIndex) {
				nextDelim = dotIndex;
				dotIndex = -1;
			} else if (openingbracketIndex < dotIndex
					&& openingbracketIndex < closingbracketIndex) {
				nextDelim = openingbracketIndex;
				openingbracketIndex = -1;
			} else if (closingbracketIndex < dotIndex
					&& closingbracketIndex < openingbracketIndex) {
				if (closingbracketIndex == (objectReference.length() - 1)) {
					nodeNames.add(objectReference.substring(lastDelim + 1,
							closingbracketIndex));
					return;
				}
				nextDelim = closingbracketIndex;
				closingbracketIndex = -1;
			}

			nodeNames.add(objectReference.substring(lastDelim + 1, nextDelim));

		}
	}

}

==> openiec61850-0.12.0.tgz: openiec61850/src/common/src/main/java/org/openiec61850/common/model/ObjectReference.java <==
/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * ObjectReference syntax: LDName/LNName.DOName[.Name[. ...]]
 * 
 * @author Michael Zillgith
 * @author Tobias Weidelt
 * @author Stefan Feuerhahn
 * 
 */
public class ObjectReference implements Iterable<String> {

	private final String objectReference;
	private List<String> nodeNames = null;
	private String nameCache = null;

	public ObjectReference(String objectReference) {
		if (objectReference == null || objectReference == "") {
			throw new IllegalArgumentException();
		}
		this.objectReference = objectReference;
	}

	/**
	 * @return Returns name part of the reference.
	 */
	public String getName() {
		if (nameCache != null) {
			return nameCache;
		}

		int firstSlash = objectReference.indexOf('/');

		if (firstSlash >= 0) {
			int lastDot = objectReference.lastIndexOf('.');
			if (lastDot > firstSlash) {
				// int lastSqrBracket = objectReference.lastIndexOf('[');
				// if (lastSqrBracket != -1) {//this is for nested values
				// if (lastSqrBracket > lastDot) {
				// int lastSqrClosingBracket = objectReference
				// .lastIndexOf(']');
				// nameCache = objectReference.substring(
				// lastSqrBracket + 1, lastSqrClosingBracket);
				// return nameCache;
				// }
				// } else {// this is for arrays
				int lastBracket = objectReference.lastIndexOf('(');
				if (lastBracket > lastDot) {
					int lastClosingBracket = objectReference.lastIndexOf(')');
					nameCache = objectReference.substring(lastBracket + 1, lastClosingBracket);
					return nameCache;
				}
				// }
				nameCache = objectReference.substring(lastDot + 1);
				return nameCache;

			}
			nameCache = objectReference.substring(firstSlash + 1);
			return nameCache;
		}
		nameCache = objectReference;
		return nameCache;
	}

	@Override
	public String toString() {
		return objectReference;
	}

	public boolean isLogicalDeviceRef() {
		return (size() == 1);
	}

	public boolean isLogicalNodeRef() {
		return (size() == 2);
	}

	public boolean isDataRef() {
		return (size() > 2);
	}

	public Iterator<String> iterator() {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.iterator();
	}

	public String get(int i) {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.get(i);
	}

	public int size() {
		if (nodeNames == null) {
			parseForNameList();
		}
		return nodeNames.size();
	}

	public String getParentRef() {
		int lastDot = objectReference.lastIndexOf('.');
		if (lastDot >= 0) {
			return objectReference.substring(0, lastDot);
		}
		int lastSlash = objectReference.lastIndexOf('/');
		if (lastSlash >= 0) {
			return objectReference.substring(0, lastSlash);
		}
		return "";
	}

	private void parseForNameList() {

		nodeNames = new LinkedList<String>();

		int lastDelim = -1;
		int nextDelim = objectReference.indexOf('/');
		if (nextDelim == -1) {
			nodeNames.add(objectReference.substring(lastDelim + 1, objectReference.length()));
			return;
		}

		nodeNames.add(objectReference.substring(lastDelim + 1, nextDelim));

		int dotIndex = -1;
		int openingbracketIndex = -1;
		int closingbracketIndex = -1;
		while (true) {
			lastDelim = nextDelim;
			if (dotIndex == -1) {
				dotIndex = objectReference.indexOf('.', lastDelim + 1);
				if (dotIndex == -1) {
					dotIndex = objectReference.length();
				}
			}
			if (openingbracketIndex == -1) {
				openingbracketIndex = objectReference.indexOf('(', lastDelim + 1);
				if (openingbracketIndex == -1) {
					openingbracketIndex = objectReference.length();
				}
			}
			if (closingbracketIndex == -1) {
				closingbracketIndex = objectReference.indexOf(')', lastDelim + 1);
				if (closingbracketIndex == -1) {
					closingbracketIndex = objectReference.length();
				}
			}

			if (dotIndex == openingbracketIndex && dotIndex == closingbracketIndex) {
				nodeNames.add(objectReference.substring(lastDelim + 1, objectReference.length()));
				return;
			}

			if (dotIndex < openingbracketIndex && dotIndex < closingbracketIndex) {
				nextDelim = dotIndex;
				dotIndex = -1;
			}
			else if (openingbracketIndex < dotIndex && openingbracketIndex < closingbracketIndex) {
				nextDelim = openingbracketIndex;
				openingbracketIndex = -1;
			}
			else if (closingbracketIndex < dotIndex && closingbracketIndex < openingbracketIndex) {
				if (closingbracketIndex == (objectReference.length() - 1)) {
					nodeNames.add(objectReference.substring(lastDelim + 1, closingbracketIndex));
					return;
				}
				nextDelim = closingbracketIndex;
				closingbracketIndex = -1;
			}

			nodeNames.add(objectReference.substring(lastDelim + 1, nextDelim));

		}
	}

}
